#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
django_react_templatetags
----------
This extension allows you to add React components into your django templates.
"""

__title__ = "django_react_templatetags"
__version__ = "6.0.1"
__build__ = 600
__author__ = "Martin Sandström"
__license__ = "MIT"
__copyright__ = "Copyright 2015-2020 Fröjd Interactive"
